package jalview.schemes;

import static org.testng.Assert.assertEquals;

import java.awt.Color;

import jalview.datamodel.AlignmentAnnotation;
import jalview.datamodel.Annotation;
import jalview.datamodel.GraphLine;

import org.testng.annotations.BeforeClass;
import org.testng.annotations.Test;

public class AnnotationColourGradientTest
{
  final static int WIDTH = 11;

  final static int THRESHOLD = 5;

  private AlignmentAnnotation ann;

  Color minColour = new Color(50, 200, 150);

  Color maxColour = new Color(150, 100, 250);

  @BeforeClass
  public void setUp()
  {
    Annotation[] anns = new Annotation[WIDTH];
    /*
     * set annotations with values 0-10
     */
    for (int col = 0; col < WIDTH; col++)
    {
      anns[col] = new Annotation("a", "a", 'a', col);
    }

    /*
     * AlignmentAnnotation constructor works out min-max range
     */
    ann = new AlignmentAnnotation("", "", anns);
    ann.setThreshold(new GraphLine(THRESHOLD, "", Color.RED));
  }

  @Test(groups = "Functional")
  public void testShadeCalculation_noThreshold()
  {
    AnnotationColourGradient testee = new AnnotationColourGradient(ann,
            minColour, maxColour, AnnotationColourGradient.NO_THRESHOLD);
    for (int col = 0; col < WIDTH; col++)
    {
      Color result = testee.shadeCalculation(ann, col);
      /*
       * column <n> is n/10 of the way from minCol to maxCol
       */
      Color expected = new Color(50 + 10 * col, 200 - 10 * col,
              150 + 10 * col);
      assertEquals(result, expected, "for column " + col);
    }
  }

  /**
   * Test the 'colour above threshold' case
   */
  @Test(groups = "Functional")
  public void testShadeCalculation_aboveThreshold()
  {
    AnnotationColourGradient testee = new AnnotationColourGradient(ann,
            minColour, maxColour, AnnotationColourGradient.ABOVE_THRESHOLD);
    for (int col = 0; col < WIDTH; col++)
    {
      Color result = testee.shadeCalculation(ann, col);
      /*
       * colour is derived regardless of the threshold value 
       * (the renderer that will suppress colouring if
       * above/below threshold)
       */
      Color expected = new Color(50 + 10 * col, 200 - 10 * col,
              150 + 10 * col);
      assertEquals(result, expected, "for column " + col);
    }

    /*
     * now make 6-10 the span of the colour range
     * (annotation value == column number in this test)
     */
    testee.setThresholdIsMinMax(true);
    for (int col = 0; col < THRESHOLD; col++)
    {
      /*
       * colours below the threshold are computed as before
       */
      Color expected = new Color(50 + 10 * col, 200 - 10 * col,
              150 + 10 * col);
      Color result = testee.shadeCalculation(ann, col);
      assertEquals(result, expected, "for column " + col);
    }
    for (int col = THRESHOLD; col < WIDTH; col++)
    {
      /*
       * colours for values >= threshold are graduated
       * range is 6-10 so steps of 100/5 = 20
       */
      int factor = col - THRESHOLD;
      Color expected = new Color(50 + 20 * factor, 200 - 20 * factor,
              150 + 20 * factor);
      Color result = testee.shadeCalculation(ann, col);
      assertEquals(result, expected, "for column " + col);
    }
  }

  /**
   * Test the 'colour below threshold' case
   */
  @Test(groups = "Functional")
  public void testShadeCalculation_belowThreshold()
  {
    /*
     * change threshold to 5 so we have an easy calculation of
     * the min-max range 0-5
     */
    int threshold = 5;
    ann.setThreshold(new GraphLine(threshold, "", Color.RED));
    AnnotationColourGradient testee = new AnnotationColourGradient(ann,
            minColour, maxColour, AnnotationColourGradient.BELOW_THRESHOLD);

    for (int col = 0; col < WIDTH; col++)
    {
      Color result = testee.shadeCalculation(ann, col);
      /*
       * colour is derived regardless of the threshold value 
       * (the renderer that will suppress colouring if
       * above/below threshold)
       */
      Color expected = new Color(50 + 10 * col, 200 - 10 * col,
              150 + 10 * col);
      assertEquals(result, expected, "for column " + col);
    }

    /*
     * now make 0-5 the span of the colour range
     * (annotation value == column number in this test)
     */
    testee.setThresholdIsMinMax(true);
    for (int col = threshold + 1; col < WIDTH; col++)
    {
      /*
       * colours above the threshold are computed as before
       */
      Color expected = new Color(50 + 10 * col, 200 - 10 * col,
              150 + 10 * col);
      Color result = testee.shadeCalculation(ann, col);
      assertEquals(result, expected, "for column " + col);
    }

    for (int col = 0; col <= threshold; col++)
    {
      /*
       * colours for values <= threshold are graduated
       * range is 0-5 so steps of 100/5 = 20
       */
      Color expected = new Color(50 + 20 * col, 200 - 20 * col,
              150 + 20 * col);
      Color result = testee.shadeCalculation(ann, col);
      assertEquals(result, expected, "for column " + col);
    }
  }
}
